/*****************************************************************************
*                                                                            *
* void      DCT(int Filterung)                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Diskrete Kosinus-Transformation mit Filterung und Rcktransformation       *
* DCT im 8x8-Fenster fuer JPEG                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 04.02.1993                                               *
* Letzte Aenderung: 14.05.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:  Filterung = TRUE : Filterung durchfuehren                          *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Save_PCX()                                                     *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define FELDGROESSE  8


void DCT(int Filterung)
{
  int BildNr1, BildNr2;                 /* Nummer von Quell- und Ziel-Bild  */
  long Index, Index1, Index2;           /* Indizes auf die Pixel der Bilder */
  long z,s,m,n,k,l;                     /* Laufvariablen fuer DCT-Formel    */
  long Spaltenzahl;
  int Anzahl, Kantenlaenge;
  float Zwerg, Maximum, Minimum, Normierung;
  float DCT_Werte[FELDGROESSE][FELDGROESSE]; /* Werte der DC-Transformation */
  float COS_Werte[FELDGROESSE][FELDGROESSE]; /* Tabelle zur schnellen Cos-B.*/
  FILE *fp_DCT;                         /* Pointer auf Hilfsdatei  DCT.tmp  */

  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* DCT - 8x8-Felder fuer JPEG *");

  gotoxy(35,20);
  BildNr1=Read_Int("Quell-Bild ", 1, ':');
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used) return;

  gotoxy(35,21);
  BildNr2=Read_Int("Ziel-Bild  ", 2, ':');
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  Index1 = (long) (BildNr1) * PIC_SIZE;  /* Anfangsadressen der beiden ...  */
  Index2 = (long) (BildNr2) * PIC_SIZE;  /* Anfangsadressen der beiden ...  */

  Anzahl = (Bild[BildNr1].Zeilen/FELDGROESSE); /* Zeilen == Spalten */

  if ( (Anzahl*FELDGROESSE != Bild[BildNr1].Spalten) ||
       (Anzahl*FELDGROESSE != Bild[BildNr1].Zeilen) ) {
     ShowError("Falsche Aufloesung des Bildes");
     return;
  }

  Spaltenzahl=(long)Bild[BildNr1].Spalten;
  Maximum= -1000;                 /* Minimum/Maximum zur Grauwertnormierung */
  Minimum=  1000;

  /* Zur schnellen Berechnung der Cos-Terme in der Transformation werden    */
  /* diese zuerst berechnet und in einer Tabelle abgelegt.                  */
  for (k=0; k<FELDGROESSE; k++) {
      for (m=0; m<FELDGROESSE; m++) {
          COS_Werte[m][k] =  cos(((float)(m+m+1)*k*PI)/(2.0*FELDGROESSE));
      }
  }

  /* Hilfsdatei  DCT.tmp  im Binaer-Modus neu oeffnen/anlegen               */
  fp_DCT=fopen("DCT.tmp", "wb+");
  if (fp_DCT==NULL) {
     ShowError("Temp.-Datei konnte nicht angelegt werden!");
     fclose(fp_DCT);
     unlink("DCT.tmp");
     return;
  }

  Normierung = 2.0 / (float)FELDGROESSE;

  WriteText(35,23,"Transformation in den Frequenzraum");
  WriteText(35,24,"Feld        ");
  for (z=0; z<Anzahl; z++) {            /* Transformation ueber alle Felder */
      for (s=0; s<Anzahl; s++) {
          gotoxy(42,24);
          WriteInt((int)(z*Anzahl + s + 1));
          Index = Index1 + z*FELDGROESSE*Spaltenzahl + s*FELDGROESSE;
          for (k=0; k<FELDGROESSE; k++) {           /* ueber alle DCT-Werte */
              for (l=0; l<FELDGROESSE; l++) {
                  Zwerg = 0.0;
                  for (m=0; m<FELDGROESSE; m++) {   /* ueber alle Pixel     */
                      for (n=0; n<FELDGROESSE; n++) {
                          Zwerg += (float)Pictures[Index + m*Spaltenzahl + n] *
                                   COS_Werte[m][k] * COS_Werte[n][l];
                          /*  cos(((float)(m+m+1)*k*PI)/(2.0*FELDGROESSE)) *
                              cos(((float)(n+n+1)*l*PI)/(2.0*FELDGROESSE)); */
                      }
                  }
                  DCT_Werte[k][l]=Zwerg*Normierung; /* Normierung==2/N;N==8 */
              }
          }
          for (k=0; k<FELDGROESSE; k++) {
              DCT_Werte[k][0] *= 0.7071067812;/* Falls Index==0; *1/sqrt(2) */
              DCT_Werte[0][k] *= 0.7071067812;/* Falls Index==0; *1/sqrt(2) */

              /* sofort Minimum/Maximum zur Normierung berechnen            */
              for (l=0; l<FELDGROESSE; l++) {
                  if      (DCT_Werte[k][l]>Maximum) Maximum=DCT_Werte[k][l];
                  else if (DCT_Werte[k][l]<Minimum) Minimum=DCT_Werte[k][l];

              }
          }
          /* komplettes Feld mit den berechneten DCT-Werten in Hilfsdatei   */
          for (k=0; k<FELDGROESSE; k++) {
              fwrite(DCT_Werte[k], sizeof(float), FELDGROESSE, fp_DCT);
          }
      }
  }

  /* genauer Normierungs-Faktor berechnen. Achtung, falls Minimum==Maximum  */
  if (Minimum == Maximum) Normierung=1.0;
  else                    Normierung=255.0/(Maximum-Minimum);

  /* Nun im Grauwert normiertes DCT-Bild anzeigen. Werte aus DCT.tmp lesen. */
  WriteText(35,23,"Normierung der DCT-Werte          ");
  WriteText(35,24,"            ");
  rewind(fp_DCT);
  for (z=0; z<Anzahl; z++) {
      for (s=0; s<Anzahl; s++) {
          Index = Index2 + z*FELDGROESSE*Spaltenzahl + s*FELDGROESSE;
          for (k=0; k<FELDGROESSE; k++) {
              fread(DCT_Werte[k], sizeof(float), FELDGROESSE, fp_DCT);
          }

          for (k=0; k<FELDGROESSE; k++) {
              for (l=0; l<FELDGROESSE; l++) {
                  Pictures[Index + k*Spaltenzahl + l] =
                               (int)((DCT_Werte[k][l] - Minimum) * Normierung);
              }
          }
      }
  }

  /* Jetzt die Bildinformation fuer das neue Bild korrekt setzen            */
  Bild[BildNr2].Zeilen =Bild[BildNr1].Zeilen;
  Bild[BildNr2].Spalten=Bild[BildNr1].Spalten;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);


  /*----------------------- Filterung/Ruecktransformation ------------------*/
  if (Filterung) {
     rewind(fp_DCT);
     ClearTextWindow(35,22,80,25);
     /* Kantenlaenge der Matrix, d.h. welcher linker obere Teil von den     */
     /* 8x8-DCT-Werten verwendet werden sollen. Alle anderen auf Null setzen*/
     Kantenlaenge=-1;
     while ( (Kantenlaenge>FELDGROESSE) || (Kantenlaenge<0) ) {
        gotoxy(35,22);
        Kantenlaenge=Read_Int("Groesse der Matrix (1x1...8x8) ", 8, ':');
     }

     Normierung = 2.0 / (float)FELDGROESSE;

     WriteText(35,23,"Ruecktransformation in den Ortsraum");
     WriteText(35,24,"Feld        ");
     for (z=0; z<Anzahl; z++) {         /* Transformation ueber alle Felder */
         for (s=0; s<Anzahl; s++) {
             gotoxy(42,24);
             WriteInt((int)(z*Anzahl + s + 1));
             Index = Index2 + z*FELDGROESSE*Spaltenzahl + s*FELDGROESSE;
             for (k=0; k<FELDGROESSE; k++) {
                 fread(DCT_Werte[k], sizeof(float), FELDGROESSE, fp_DCT);
             }
             for (m=0; m<FELDGROESSE; m++) {
                 for (n=0; n<FELDGROESSE; n++) {
                     Zwerg = 0.0;
                     for (k=0; k<Kantenlaenge; k++) {
                         for (l=0; l<Kantenlaenge; l++) {
                             /* Skalierung bei Index==0 beachten (1/sqrt(2))*/
                             Zwerg += ((k==0) ? 0.7071067812:1.0)*
                                      ((l==0) ? 0.7071067812:1.0)*
                                   DCT_Werte[k][l]*
                                   COS_Werte[m][k] * COS_Werte[n][l];
                          /*  cos(((float)(m+m+1)*k*PI)/(2.0*FELDGROESSE)) *
                              cos(((float)(n+n+1)*l*PI)/(2.0*FELDGROESSE)); */
                         }
                     }
                     Zwerg *=Normierung;         /* Normierung == 2/N; N==8 */
                     /* Achtung bei Unter- oder Ueberschreitung des Bereichs*/
                     Pictures[Index + m*Spaltenzahl + n] =
                             (Zwerg>255.0) ? WEISS:( (Zwerg<0.0) ? SCHWARZ:(int)Zwerg );
                 }
             }
         }
     }

     Make_Icon(BildNr2);
     Show_Pic(BildNr2, BildNr2+1);
 }

 fclose(fp_DCT);      /* temporaere Datei DCT.tmp wieder schliessen und ... */
 unlink("DCT.tmp");   /* loeschen, da nicht mehr benoetigt.                 */

}
