/*****************************************************************************
*                                                                            *
* void      FFT(int Filterung)                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Zweidimensionale Fast-Fourier-Transformation mit Filterung (optional)      *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 13.12.1992                                               *
* Letzte Aenderung: 17.12.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input:  Filterung = TRUE : Filterung durchfuehren                          *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Make_Icon(), Save_Pic()                                        *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void FFT(int Filterung)
{
  int BildNr1, BildNr2;                   /* Nummern der verwendeten Bilder */
  int i,j, Zwerg;
  int Tiefpass, Kantenlaenge, Randbereich;
  long Index, Index1, Index2;
  int Anzahl, ld_Anzahl, Anzahl_Halbe;
  unsigned char BitReverse, ReverseShuffel, BitTest;
  unsigned char BitReverseShuffel[S_RES_MAX]; /* Index-Feld fuer Bit-Shuffle*/
  float Faktor_C;                /* Skalierung-Faktor zur Bild-Darstellung  */
  float Dat_Real[S_RES_MAX];     /* Hilfsvektoren fuer die Zeilenergebnisse */
  float Dat_Imag[S_RES_MAX];
  float Dat_Temp[S_RES_MAX];
  FILE *fp_Imag1, *fp_Real1, *fp_Imag2, *fp_Real2; /* "Speichererweiterung" */

  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* FFT - zweidimensional *");

  gotoxy(35,20);
  BildNr1=Read_Int("Quell-Bild ", 1, ':');
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used) return;

  gotoxy(35,21);
  BildNr2=Read_Int("Ziel-Bild  ", 2, ':');
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  Index1 = (long) (BildNr1) * PIC_SIZE;  /* Anfangsadressen der beiden ...  */
  Index2 = (long) (BildNr2) * PIC_SIZE;  /* ... Bilder.                     */

  Anzahl = Bild[BildNr1].Zeilen;         /* Nur quadrat.  Zeilen == Spalten */
  Anzahl_Halbe = Anzahl>>1;
  ld_Anzahl = 0;
  while (Anzahl > (1<<ld_Anzahl)) ld_Anzahl++;

  /* Pruefen der Aufloesung. Diese muss eine 2er-Potenz sein.               */
  if ( (Anzahl != Bild[BildNr1].Spalten) ||
       (Anzahl != (1<<ld_Anzahl)) ) {
     ShowError("Falsche Aufloesung des Bildes");
     return;
  }

  Faktor_C = 10*(float)WEISS / log10((float) Anzahl + 1.0);

  /* Feld mit Bit-Reverse-Shuffling-Indizes erstellen                      */
  for (i=0; i<Anzahl; i++) {
      ReverseShuffel=0;
      BitReverse= (1<<(ld_Anzahl-1));
      BitTest   =1;
      for (j=0; j<ld_Anzahl; j++) {
          if (i & BitTest) ReverseShuffel = ReverseShuffel | BitReverse;
          BitTest    <<=1;
          BitReverse >>=1;
      }
      BitReverseShuffel[i]=ReverseShuffel;
  }

  fp_Imag1=fopen("Imag1.tmp", "wb+"); /* Dateien mit dem temporaeren Ergebn.*/
  fp_Real1=fopen("Real1.tmp", "wb+");
  fp_Imag2=fopen("Imag2.tmp", "wb+");
  fp_Real2=fopen("Real2.tmp", "wb+");

  if ( (fp_Imag1==NULL) || (fp_Imag2==NULL) ||
       (fp_Real1==NULL) || (fp_Real2==NULL) ) {
     ShowError("Temp.-Datei konnte nicht angelegt werden!");
     fclose(fp_Real1);
     fclose(fp_Real2);
     fclose(fp_Imag1);
     fclose(fp_Imag2);

     unlink("Real1.tmp");
     unlink("Real2.tmp");
     unlink("Imag1.tmp");
     unlink("Imag2.tmp");
     return;
  }

  /* Zuerst zeilenweise Fourier-Transformieren ...                          */
  WriteText(35,23,"Transformation in den Frequenzraum");
  WriteText(35,24,"Zeile       ");
  for (i=0; i<Anzahl; i++) {
      gotoxy(42,24);
      WriteInt(i);
      Index = Index1 + (long)i * (long)Anzahl;
      for (j=0; j<Anzahl; j++) {
          Dat_Real[ BitReverseShuffel[j] ] = Pictures[Index + (long)j];
          Dat_Imag[ BitReverseShuffel[j] ] = 0.0;
      }
      FFT_1Dim(TRUE, Anzahl, ld_Anzahl, Dat_Real, Dat_Imag);

      fwrite(Dat_Real, sizeof(float), Anzahl, fp_Real1);
      fwrite(Dat_Imag, sizeof(float), Anzahl, fp_Imag1);
  }

  /* ... und das Ergebnis dann spaltenweise Fourier-Transformation          */
  WriteText(35,24,"Spalte      ");
  for (i=0; i<Anzahl; i++) {
      gotoxy(42,24);
      WriteInt(i);
      rewind(fp_Real1);
      rewind(fp_Imag1);
      for (j=0; j<Anzahl; j++) {
          fread(Dat_Temp, sizeof(float), Anzahl, fp_Real1);
          Dat_Real[ BitReverseShuffel[j] ] = Dat_Temp[i];
          fread(Dat_Temp, sizeof(float), Anzahl, fp_Imag1);
          Dat_Imag[ BitReverseShuffel[j] ] = Dat_Temp[i];
      }
      FFT_1Dim(TRUE, Anzahl, ld_Anzahl, Dat_Real, Dat_Imag);

      fwrite(Dat_Real, sizeof(float), Anzahl, fp_Real2);
      fwrite(Dat_Imag, sizeof(float), Anzahl, fp_Imag2);

      for (j=0; j<Anzahl; j++) {
          Zwerg = (int)(Faktor_C *
                        log10(1.0 + sqrt(Dat_Real[j]*Dat_Real[j] +
                                         Dat_Imag[j]*Dat_Imag[j])) );
          if (Zwerg<SCHWARZ)    Zwerg=SCHWARZ;
          else if (Zwerg>WEISS) Zwerg=WEISS;
          /* Umordnung der Werte zur gewohnten symmetrischen Anordnung      */
          if (i<Anzahl_Halbe)
             Index = Index2 + (long)(i+Anzahl_Halbe) * (long)Anzahl + (long)j +
                     ( (j<Anzahl_Halbe) ? (long)Anzahl_Halbe : (long)(-Anzahl_Halbe) );
          else            /* if (i>Anzahl_Halbe) */
             Index = Index2 + (long)(i-Anzahl_Halbe) * (long)Anzahl + (long)j +
                     ( (j<Anzahl_Halbe) ? (long)Anzahl_Halbe : (long)(-Anzahl_Halbe) );
          Pictures[Index]=Zwerg;
      }
  }

  /* Jetzt die Bildinformation fuer das neue Bild korrekt setzen            */
  Bild[BildNr2].Zeilen =Anzahl;
  Bild[BildNr2].Spalten=Anzahl;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);

  fclose(fp_Real1);
  fclose(fp_Imag1);
  unlink("Real1.tmp");
  unlink("Imag1.tmp");
  fp_Imag1=fopen("Imag1.tmp", "wb+");
  fp_Real1=fopen("Real1.tmp", "wb+");
  if ( (fp_Imag1==NULL) || (fp_Real1==NULL) ) {
     ShowError("Temp.-Datei konnte nicht angelegt werden!");
     Filterung=FALSE;
  }

  /*----------------------- Filterung/Ruecktransformation ------------------*/
  if (Filterung) {
     rewind(fp_Real2);
     rewind(fp_Imag2);
     Index2 = (long) (BildNr2) * PIC_SIZE;

     ClearTextWindow(35,23,80,25);
     gotoxy(35,23);
     Tiefpass=(Upper(Read_Char("Tiefpass-Filterung (J/N)", 'J'))=='J');
     Kantenlaenge=-1;
     while ( (Kantenlaenge>Anzahl) || (Kantenlaenge<0) ) {
        gotoxy(35,24);
        Kantenlaenge=Read_Int("Rechteckfilter-Kantenlaenge ", (int)(Anzahl/2), '?');
     }
     Randbereich=(Anzahl-Kantenlaenge)/2;

     /* Frequenz + Filter anzeigen */
     for (i=0; i<Anzahl; i++) {
         Index = Index2 + (long)i * (long)Anzahl;
         for (j=0; j<Anzahl; j++) {
             if (Tiefpass) {
                if ( (i<Randbereich) || (i>=(Anzahl-Randbereich)) ||
                     (j<Randbereich) || (j>=(Anzahl-Randbereich)) )
                   Pictures[Index + (long)j]=SCHWARZ;
             }
             else {
                if ( (i>=Randbereich) && (i<(Anzahl-Randbereich)) &&
                     (j>=Randbereich) && (j<(Anzahl-Randbereich)) )
                   Pictures[Index + (long)j]=SCHWARZ;
             }
         }
     }
     Make_Icon(BildNr2);
     Show_Pic(BildNr2, BildNr2+1);

     Randbereich=Kantenlaenge/2;

     /* Ruecktransformation zuerst wieder zeilenweise ...                   */
     ClearTextWindow(35,23,80,25);
     WriteText(35,23,"Transformation in den Ortsraum          ");
     WriteText(35,24,"Zeile       ");
     for (i=0; i<Anzahl; i++) {
         gotoxy(42,24);
         WriteInt(i);
         fread(Dat_Temp, sizeof(float), Anzahl, fp_Real2);
         for (j=0; j<Anzahl; j++) {
             if ( ((i<Randbereich) && (j<Randbereich))           ||
                  ((i>=(Anzahl-Randbereich)) && (j<Randbereich)) ||
                  ((i<Randbereich) && (j>=(Anzahl-Randbereich))) ||
                  ((i>=(Anzahl-Randbereich)) && (j>=(Anzahl-Randbereich))) )
                Dat_Real[ BitReverseShuffel[j] ] = (Tiefpass) ? Dat_Temp[j] : 0.0;
             else
                Dat_Real[ BitReverseShuffel[j] ] = (Tiefpass) ? 0.0 : Dat_Temp[j];
         }

         fread(Dat_Temp, sizeof(float), Anzahl, fp_Imag2);
         for (j=0; j<Anzahl; j++) {
             if ( ((i<Randbereich) && (j<Randbereich))           ||
                  ((i>=(Anzahl-Randbereich)) && (j<Randbereich)) ||
                  ((i<Randbereich) && (j>=(Anzahl-Randbereich))) ||
                  ((i>=(Anzahl-Randbereich)) && (j>=(Anzahl-Randbereich))) )
                Dat_Imag[ BitReverseShuffel[j] ] = (Tiefpass) ? Dat_Temp[j] : 0.0;
             else
                Dat_Imag[ BitReverseShuffel[j] ] = (Tiefpass) ? 0.0 : Dat_Temp[j];
         }
         FFT_1Dim(FALSE, Anzahl, ld_Anzahl, Dat_Real, Dat_Imag);

         fwrite(Dat_Real, sizeof(float), Anzahl, fp_Real1);
         fwrite(Dat_Imag, sizeof(float), Anzahl, fp_Imag1);
     }
  
     /* ... und danach wieder spaltenweise                                  */
     WriteText(35,24,"Spalte      ");
     for (i=0; i<Anzahl; i++) {
         gotoxy(42,24);
         WriteInt(i);
         rewind(fp_Real1);
         rewind(fp_Imag1);
         for (j=0; j<Anzahl; j++) {
             fread(Dat_Temp, sizeof(float), Anzahl, fp_Real1);
             Dat_Real[ BitReverseShuffel[j] ] = Dat_Temp[i];
             fread(Dat_Temp, sizeof(float), Anzahl, fp_Imag1);
             Dat_Imag[ BitReverseShuffel[j] ] = Dat_Temp[i];
         }
         FFT_1Dim(FALSE, Anzahl, ld_Anzahl, Dat_Real, Dat_Imag);

         Index = Index2 + (long)Anzahl * (long)i;
         for (j=0; j<Anzahl; j++) {
             Zwerg=(int) (Dat_Real[j] + 0.5);
             if (Zwerg<SCHWARZ)    Zwerg=SCHWARZ;
             else if (Zwerg>WEISS) Zwerg=WEISS;
             Pictures[Index+(long)j]=Zwerg;
         }
     }
     Make_Icon(BildNr2);
     Show_Pic(BildNr2, BildNr2+1);
 }

 fclose(fp_Real1);      /* Verwendete temporaere Dateien schliessen ...     */
 fclose(fp_Real2);
 fclose(fp_Imag1);
 fclose(fp_Imag2);

 unlink("Real1.tmp");   /* ... und loeschen.                                */
 unlink("Real2.tmp");
 unlink("Imag1.tmp");
 unlink("Imag2.tmp");

}
