/*****************************************************************************
*                                                                            *
* void      Marr_Hildreth(int BildNr1, int BildNr2, int Radius, float Sigma) *
*                                                                            *
******************************************************************************
*                                                                            *
* Marr-Hildreth  Kanten-Operator    (Integer-Version)                        *
* Literatur:  D. Marr, E. Hildreth "Theory of edge detection"                *
*             Proc. R. Soc. Lond. B 207, pp.187-217, (1980)                  *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 22.04.1993                                               *
* Letzte Aenderung: 23.04.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr1 = Quelbild                                                  *
*        BildNr2 = Zielbild (Gradientenbild)                                 *
*        Radius  = Groesse des Filterkerns (1 ... 20)                        *
*        Sigma   = Standardabweichung (1.0 ... 5.0)                          *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(),                    *
*             Geduld(), Make_Icon(), Show_Pic()                              *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"

#define FAKTOR -200.0    /* FAKTOR <= -255; -200.0 ist daher ein guter Wert */


void Marr_Hildreth(int BildNr1, int BildNr2, int Radius, float Sigma)
{
  int  Matrix_MH[41][41];         /* Matrix fuer Marr-Hildreth Operator     */
  int i,j,x,y;
  double Sigma_Quadrat, Zwerg;    /* Hilfsvariablen zur Berechnung der ...  */
  double Abstand, Zaehler, Nenner;/* ... Matrix fuer den Marr-Hildreth ...  */
  double Normierung;              /* ... Operator (LoG)                     */
  long  z, s, Z_Index, S_Index;   /* Variablen fuer Zeilen und Spaltenindex */
  long  Index1, Index2;           /* Adresse der beiden Bilder              */
  long  Zmax, Smax;               /* Zeilen-/Spaltengroesse der Bilder      */
  long  Zwei_Zmax_1, Zwei_Smax_1, Wert;
  FILE  *fp_MH;                   /* File-Pointer fuer die LoG-Matrixdatei  */


  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* Anwendung des Marr-Hildreth-Operators *");

  if (!BildNr1) {
     gotoxy(35,20);
     BildNr1=Read_Int("Quell-Bild      ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr1<=0) || (BildNr1>MAX_PICTURE)) return;
  BildNr1--;                          /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr1].Used)   return;

  if (!BildNr2) {
     gotoxy(35,21);
     BildNr2=Read_Int("Gradienten-Bild ", 2, ':');
  }
  /* Gueltige Bild-Nummer eingegeben?                                       */
  if ((BildNr2<=0) || (BildNr2>MAX_PICTURE)) return;
  BildNr2--;                          /* die Zaehlung faengt immer bei 0 an */

  while ( (Sigma<1.0) || (Sigma>5.0) ) {
     gotoxy(35,22);
     Sigma=Read_Float("Standardabweichung (1.0-5.0) ", 1.5, ':');
  }

  /* Sinnvoller Radius=3*2*sqrt(2)*Sigma  (3w bis 4w mit w=2*sqrt(2)*Sigma) */
  while ( (Radius<1) || (Radius>20) ) {
     gotoxy(35,23);
     Radius=Read_Int("Radius des Filters (1-20)      ", (int)(8.4*Sigma-1.0)/2.0, ':');
  }

  Geduld();

  Index1 = (long) (BildNr1) * PIC_SIZE; /* Adressen der beiden Bilder       */
  Index2 = (long) (BildNr2) * PIC_SIZE;

  Zmax = Bild[BildNr1].Zeilen;
  Smax = Bild[BildNr1].Spalten;
  Zwei_Zmax_1 = 2*Zmax - 1;
  Zwei_Smax_1 = 2*Smax - 1;


  Zaehler=0.0;
  Nenner =0.0;
  /* Die Filtermaske ist nun mit den Werten zu initialisieren. Dazu eine    */
  /* Normierung durchfuehren, so dass die Summe der Werte etwa gleich Null. */
  Sigma_Quadrat=Sigma*Sigma;
  for (i= -Radius; i<=Radius; i++)
      for (j= -Radius; j<=Radius; j++) {
          Abstand  = (double)(i*i + j*j);
          Zaehler += exp( -Abstand/(2.0*Sigma_Quadrat));
          Nenner  += (Abstand/(2.0*Sigma_Quadrat)) * exp( -Abstand/(2.0*Sigma_Quadrat));
      }

  if (Nenner!=0.0) Normierung=Zaehler/Nenner;
  else             Normierung=0.0;

  for (i= -Radius; i<=Radius; i++)
      for (j= -Radius; j<=Radius; j++) {
          Abstand = (double)(i*i + j*j);
          Zwerg   = FAKTOR * (Normierung*Abstand/(2.0*Sigma_Quadrat) - 1.0) *
                              exp(-Abstand/(2.0*Sigma_Quadrat));
          Matrix_MH[i+Radius][j+Radius]=
                   (Zwerg>0.0) ? (int)(Zwerg+0.5):(int)(Zwerg-0.5);
      }

  /* LoG-Maske zur Information in eine Datei MH_MATRX.DAT schreiben         */
  if ((fp_MH=fopen("MH_MATRX.DAT", "w"))!=NULL) {
     fprintf(fp_MH, "Eintraege in der normierten Matrix des Marr-Hildreth-Operators\n");
     fprintf(fp_MH, "\nFAKTOR =%4d\n", (int)FAKTOR);
     fprintf(fp_MH, "Sigma  =%4.1f\n", Sigma);
     fprintf(fp_MH, "Radius =%4d\n\n", Radius);
     for (i= -Radius; i<=Radius; i++) {
         for (j= -Radius; j<=Radius; j++) {
             fprintf(fp_MH,"%4d ", Matrix_MH[i+Radius][j+Radius]);
         }
         fprintf(fp_MH,"\n");
     }
     fclose(fp_MH);
  }

  WriteText(70,25,"Zeile=");
  for (z=0; z<Zmax; z++) {
      gotoxy(76,25);
      WriteInt((int)z);

      for (s=0; s<Smax; s++) {
          Zwerg = 0.0;
          Wert = 0.0;
          /* Ausserhalb liegende Punkte werden nach innen gespiegelt.       */
          for (y= -Radius; y<=Radius; y++) {
              Z_Index = z + (long)y;
              Z_Index = (Z_Index<0) ? (-Z_Index) :
                        ((Z_Index>=Zmax) ? (Zwei_Zmax_1-Z_Index) : Z_Index);
              Z_Index = Z_Index*Smax;
              for (x= -Radius; x<=Radius; x++) {
                  S_Index = s + (long)x;
                  S_Index = (S_Index<0) ? (-S_Index) :
                            ((S_Index>=Smax) ? (Zwei_Smax_1-S_Index) : S_Index);
                  Wert += (long)Matrix_MH[y+Radius][x+Radius] *
                          (long)Pictures[Index1 + Z_Index + S_Index];
              }
          }
          Pictures[Index2 + z*Smax + s] = (Wert<=0) ? SCHWARZ:WEISS;
      }
  }

  /* Bildinformation korrekt setzen und Zwischenergebnis anzeigen           */
  Bild[BildNr2].Zeilen =Zmax;
  Bild[BildNr2].Spalten=Smax;
  Bild[BildNr2].Used   =TRUE;
  Bild[BildNr2].Color  =FALSE;

  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);

  /* Jetzt noch die genauen Nulldurchgaenge ermitteln. Kante am Rand lassen.*/
  for (z=0; z<(Zmax-1); z++) {
      Index1=Index2 + z*Smax;
      for (s=0; s<(Smax-1); s++) {
          if      ( (Pictures[Index1]==SCHWARZ) && (Pictures[Index1+1]==WEISS) )
             Pictures[Index1]=WEISS;
          else if ( (Pictures[Index1]==WEISS)   && (Pictures[Index1+1]==SCHWARZ) )
             Pictures[Index1]=WEISS;
          else if ( (Pictures[Index1]==SCHWARZ) && (Pictures[Index1+Smax]==WEISS) )
             Pictures[Index1]=WEISS;
          else if ( (Pictures[Index1]==WEISS)   && (Pictures[Index1+Smax]==SCHWARZ) )
             Pictures[Index1]=WEISS;
          else
             Pictures[Index1]=SCHWARZ;
          Index1++;
      }
  }

  /* Rechter und unterer Rand loeschen, da diese Punkte nicht gesetzt wurden*/
  for (z=0; z<Zmax; z++) Pictures[Index2 + z*Smax + Smax -1] =SCHWARZ;
  for (s=0; s<Smax; s++) Pictures[Index2 + (Zmax-1)*Smax + s]=SCHWARZ;

  /* Nun endlich das eigentliche Kantenbild (1-Pixel breite Kanten) ausgeben*/
  Make_Icon(BildNr2);
  Show_Pic(BildNr2, BildNr2+1);

}
