/*****************************************************************************
*                                                                            *
****  Sammlung von Routinen zur I/O-, Fehlerbehandlung und zur Eingabe    ****
*                                                                            *
******************************************************************************
*                                                                            *
* int       Read_Int(char *String, int Default, char Buchstabe)              *
* float     Read_Float(char *String, float Default, char Buchstabe)          *
* void      Read_String(char Kommentar[], char String[])                     *
* char      Read_Char(char *String, char Value)                              *
* void      ShowError(char *Message)                                         *
* void      Geduld(void)                                                     *
* void      Swap(int *P1, int *P2)                                           *
* char      Upper(char c)                                                    *
* int       Add_Extension(char FileName[], char *Ext)                        *
* void      WriteText(int X_Pos, int Y_Pos, char *Text)                      *
* void      WriteInt(int Zahl)                                               *
* void      WriteFloat(float Zahl)                                           *
* void      ClearTextWindow(int X_Left, int Y_Left, int X_Right, int Y_Right)*
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


/*--------------------------------------------------------------------------*/
/* Liest fehlertolerant eine Int-Zahl ein. Ggf. wird ein Default-Wert vorg. */
int Read_Int(char *String, int Default, char Buchstabe)
{
  int  Tmp;
  char line[LINE_LENGTH];
  char Dummy[2];

  WriteText(NO_CURS_POS, NO_CURS_POS, String);
  if ( Default != INT_NO_DEFAULT ) {    /* falls Default-Wert angegeben,    */
     WriteText(NO_CURS_POS, NO_CURS_POS, "[");  /* diesen in [] ausgeben.   */
     WriteInt(Default);
     WriteText(NO_CURS_POS, NO_CURS_POS, "]");
  }
  *Dummy = Buchstabe;
  *(Dummy+1) = '\0';
  WriteText(NO_CURS_POS, NO_CURS_POS, Dummy);
  WriteText(NO_CURS_POS, NO_CURS_POS, " ");
  if ( !fgets(line, LINE_LENGTH, stdin) ) return( Default );
  if ( sscanf(line, "%d", &Tmp) != 1)     return( Default );

  return(Tmp);               /* ... ansonsten eingelesene Zahl zurueckgeben */
}

/*--------------------------------------------------------------------------*/
/* Liest fehlertolerant eine Float-Zahl ein. Ggf. wird ein Default-Wert ang.*/
float Read_Float(char *String, float Default, char Buchstabe)
{
  float Tmp;
  char line[LINE_LENGTH];
  char Dummy[2];
  float fabs();

  Tmp=FLOAT_NO_DEFAULT;
  WriteText(NO_CURS_POS, NO_CURS_POS, String);

  if (fabs(Default-Tmp)>(float)EPS_ERROR) {     /* falls Default-Wert ang.  */
     WriteText(NO_CURS_POS, NO_CURS_POS, "[");  /* diesen in [] ausgeben.   */
     WriteFloat(Default);
     WriteText(NO_CURS_POS, NO_CURS_POS, "]");
  }
  *Dummy = Buchstabe;
  *(Dummy+1) = '\0';
  WriteText(NO_CURS_POS, NO_CURS_POS, Dummy);
  WriteText(NO_CURS_POS, NO_CURS_POS, " ");
  if ( !fgets(line, LINE_LENGTH, stdin) ) return(Default);
  if ( sscanf(line, "%f", &Tmp) != 1 )    return(Default);

  return(Tmp);               /* ... ansonsten eingelesene Zahl zurueckgeben */
 }

/*--------------------------------------------------------------------------*/
/* Liest einen String von der Tastatur ein. Vorher wird Kommentar ausgegeb. */
void Read_String(char Kommentar[], char String[])
{
  fprintf(stderr,"%s",Kommentar);
  fgets(String, LINE_LENGTH, stdin);

  String[strlen(String)-1]='\0';          /* String-Abschluss mit Null-Byte */
} 

/*--------------------------------------------------------------------------*/
/* Liest ein einzelnes Zeichen von der Tastatur ein                         */
/* String = Text-Meldung                                                    */
/* Value  = Default-Wert, der mit RETURN uebernommen werden kann            */
char Read_Char(char *String, char Value)
{
  char Tmp;
  char line[LINE_LENGTH];

  fprintf(stderr,"%s [%c] ", String, Value);
  if (fgets(line, LINE_LENGTH, stdin)==NULL) return(Value);
  if (sscanf(line, "%c", &Tmp)!=1)           return(Value);

  if (Tmp != CR) return(Tmp);      /* falls das letzte Zeichen nicht RETURN */
  else           return(Value);
}
  
/*--------------------------------------------------------------------------*/
/* Zeigt eine (Fehler-) Meldung an und wartet auf eine Bestaetigung         */
void ShowError(char *Message)
{
  char Dummy[LINE_LENGTH];

  ClearTextWindow(35,24,80,25);
  WriteText(35, 24, Message);              /* Fehler-Meldungstext ausgeben */

  /* Bestaetigung und Form von RETURN-Taste abwarten                        */
  gotoxy(35, 25);
  Read_String("Bitte mit <RETURN> bestaetigen", Dummy);
}

/*--------------------------------------------------------------------------*/
/* Gibt einen Text aus                                                      */
void Geduld()
{
  WriteText(35, 25, "Bitte etwas Geduld ...                  ");
}

/*--------------------------------------------------------------------------*/
/* Vertauscht den Inhalt zweier Variablen.                                  */
void Swap(int *P1, int *P2)
{
  int Temp;

  Temp = *P1;
  *P1 = *P2;
  *P2 = Temp;
}

/*--------------------------------------------------------------------------*/
/* Verwandelt alle Kleinbuchstaben in Grossbuchstaben. Rest bleibt unveraen.*/
char Upper(char c)
{
  return( ((c>='a')&&(c<='z')) ? c-('a'-'A'):c );
} 


/*--------------------------------------------------------------------------*/
/* Falls an einem Dateinamen die Extension fehlt wird diese angehaengt.     */
int Add_Extension(char FileName[], char *Ext)
{
  if (strlen(FileName)==0)        return(0);          /* Ueberhaupt was da? */
  else if (strlen(FileName)>=strlen(Ext)) {           /* String lang genug? */
     if(strcmp(FileName+(strlen(FileName)-4),Ext) != 0) /* noch keine Exten.*/
       strcat(FileName,Ext);
  }
  else strcat(FileName,Ext);                          /* Extension anhaengen*/

  return(1);                                         /* Erfolg signalisieren*/
}

/*--------------------------------------------------------------------------*/
/* Gibt einen beliebigen Zeichenstring auf stderr aus.                      */
void WriteText(int X_Pos, int Y_Pos, char *Text)
{
  if ((X_Pos>NO_CURS_POS) && (Y_Pos>NO_CURS_POS)) gotoxy(X_Pos, Y_Pos);
  fprintf(stderr, "%s", Text);
}

/*--------------------------------------------------------------------------*/
/* Gibt eine Int-Zahl auf stderr mit minimaler Formatanpassung aus.         */
void WriteInt(int Zahl)
{
  if     ((Zahl>999) || (Zahl<0))      /* Format ueber Zahlengroesse suchen */
     fprintf(stderr, "%d", Zahl);
  else if (Zahl> 99)
     fprintf(stderr, "%3d", Zahl);
  else if (Zahl>  9)
     fprintf(stderr, "%2d", Zahl);
  else
     fprintf(stderr, "%1d", Zahl);
}

/*--------------------------------------------------------------------------*/
/* Gibt eine Float-Zahl auf stderr ohne spezielles Format aus.              */
void WriteFloat(float Zahl)
{
  if      ((Zahl>=1000) || (Zahl<=-100))
       fprintf(stderr, "%6.1f", Zahl);
  else if ((Zahl>=100) || (Zahl<=-10))
       fprintf(stderr, "%5.1f", Zahl);
  else if ((Zahl>= 10) || (Zahl<0))
       fprintf(stderr, "%4.1f", Zahl);
  else if ((Zahl>=  0) && (Zahl<10))
       fprintf(stderr, "%3.1f", Zahl);
  else
       fprintf(stderr, "%f", Zahl);
}

/*--------------------------------------------------------------------------*/
/* Loescht den angegebenen Bildschirmteil im Graphikmodus mit Leerzeichen   */
/* (X_Left, Y_Left)   = linke obere Ecke                                    */
/* (X_Right, Y_Right) = rechte untere Ecke des Fensters                     */
void ClearTextWindow(int X_Left, int Y_Left, int X_Right, int Y_Right)
{
  int i,j;

  for (i=Y_Left; i<=Y_Right; i++)
      for (j=X_Left; j<=X_Right; j++)
          WriteText(j, i, " ");

}

