/*****************************************************************************
*                                                                            *
* int       Save_PCX_Picture()                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Speichert ein Bild im PCX-Format (internes Format Nr. 5) ab.               *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 30.09.1992                                               *
* Letzte Aenderung: 10.10.1992                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Read_Int(), ShowError()        *
*             Add_Extension()                                                *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


int Save_PCX_Picture()
{
  int Z_Index, S_Index;                    /* PCX-Groessenangab. ueber Index*/
  int Anzahl;
  unsigned char PCXHeader[PCX_HEADERSIZE]; /* PCX-Bild-Header               */
  unsigned char RLC[2];                    /* Hilfsvektor fuer Run-Length-C.*/
  unsigned char LUT[768];                  /* Grauwert-Tabelle              */
  unsigned char Pixel;
  int PCX_fd;                              /* File-Descriptoren fuer Datei  */
  int i, j, z, s, BildNr;
  long Index;                              /* Adresse des Bildes            */
  char Filename[LINE_LENGTH];              /* Name Ziel-Datei (.PCX)        */
  char *PCX_Ext  =".pcx";                  /* Extension immer .PCX          */

  /*---------------------------- Dateiname eingeben ------------------------*/
  ClearTextWindow(35,18,80,25);
  WriteText(35,18,"* Bild im PCX-Format speichern *");

  gotoxy(35,20);
  BildNr=Read_Int("Welches Bild Nr. (0=Abbruch): ", 1, ' ');

  /* Gueltige Bild-Nummer eingegeben? - Und Bild auch vorhanden?            */
  if ((BildNr<=0) || (BildNr>MAX_PICTURE)) return(FALSE);
  BildNr--;                           /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr].Used) {
     ShowError("Kein Bild vorhanden.");
     return(FALSE);
  }

  gotoxy(35,22);
  Read_String("Bitte Dateinamen eingeben: ", Filename);
  if (!Add_Extension(Filename, PCX_Ext)) {
     ShowError("Fehler beim Anhaengen der PCX-Extension.");
     return(FALSE);
  }

  /* PCX-Datei neu erzeugen oder alte ueberschreiben. Daten im Binaermodus  */
  if ((PCX_fd = open(Filename, O_CREAT | O_BINARY | O_TRUNC | O_RDWR, S_IREAD | S_IWRITE)) < 0) {
     ShowError("Fehler beim Erzeugen der PCX-Datei.");
     return(FALSE);
  }

  /*---------------------------- PCX-Header erzeugen -----------------------*/
  /* Erzeuge PCX-Header mit insgesamt 128 Bytes                             */

  /* bei PCX wird nicht die Anzahl Pixel sondern Index von..bis verwendet.  */
  S_Index = Bild[BildNr].Spalten-1;
  Z_Index = Bild[BildNr].Zeilen-1;

  PCXHeader[ 0] = 10;       /* 10=0Ah = Zsoft PCX file                      */
  PCXHeader[ 1] =  5;       /* Versionsnummer; d.h. 256 Farben oder 16.7 Mio*/
  PCXHeader[ 2] =  1;       /* Encoding; 1=PCX-run-length encoding          */
  PCXHeader[ 3] =  8;       /* Bits pro Pixel und Ebene                     */
  PCXHeader[ 4] =  0;       /* Bildgroesse: Xmin     - Low-Byte, High-Byte  */
  PCXHeader[ 5] =  0;
  PCXHeader[ 6] =  0;       /* Bildgroesse: Ymin     - Low-Byte, High-Byte  */
  PCXHeader[ 7] =  0;
  PCXHeader[ 8] = S_Index-256*(S_Index/256); /* Bildgroesse: Xmax           */
  PCXHeader[ 9] = S_Index/256;
  PCXHeader[10] = Z_Index-256*(Z_Index/256); /* Bildgroesse: Ymax           */
  PCXHeader[11] = Z_Index/256;
                            /* Aufloesung: S_Res = Xmax - Xmin + 1          */
                            /*             Z_Res = Ymax - Ymin + 1          */
  PCXHeader[12] =  0;       /* Aufloesung beim Drucken; Dots/Inch in X      */
  PCXHeader[13] =150;       /* 150dpi; oft steht hier 300dpi                */
  PCXHeader[14] =  0;       /* Aufloesung beim Drucken; Dots/Inch in Y      */
  PCXHeader[15] =150;       /* 150dpi; oft steht hier 300dpi                */

  for (i=16; i<64; i++) PCXHeader[i] = 0; /* Farbtabelle fuer 16 oder wenig.*/

  PCXHeader[64] =  0;       /* Reserviert; immer 0                          */
  PCXHeader[65] =  1;       /* Anzahl der Ebenen                            */
  PCXHeader[66] = S_Index+1-256*((S_Index+1)/256); /* Benoetigte Speicher-Bytes fuer eine Zeile*/
  PCXHeader[67] = (S_Index+1)/256;
  PCXHeader[68] =  2;       /* Farbtabelle wird als Grauton interpretiert   */
  PCXHeader[69] =  0;
  PCXHeader[70] =  0;       /* Aufloesung des Bildschirms: i.d.R. unbenutzt */
  PCXHeader[71] =  0;
  PCXHeader[72] =  0;
  PCXHeader[73] =  0;

  for (i=74; i<128; i++) PCXHeader[i] = 0; /* diese Bytes sind unbenutzt    */

  for (i=0; i<256; i++)
      for (j=0; j<3; j++) LUT[i*3+j]=i;    /* in diesem Fall immer Graukeil */

  /* PCX-Header rausschreiben */
  write(PCX_fd, PCXHeader, PCX_HEADERSIZE);

  /*---------------------------- Bild-RLC berechnen ------------------------*/
  for (z=0; z<=Z_Index; z++) {
      Index = (long)z * (long)Bild[BildNr].Spalten +
                        (long)BildNr*(long)PIC_SIZE;

      s=0;
      while (s<=S_Index) {
          Pixel=Pictures[Index+s];          /* erstes Vorkommen einer Farbe */
          s++;
          Anzahl=1;
          while ((s<=S_Index) && (Pixel==Pictures[Index+s]) && (Anzahl<63)) {
              Anzahl++;
              s++;
          }
          if ((Anzahl>1) || (Pixel>=192)) { /*Anzahl>1 oder Farbe>192 in RLC*/
             RLC[0] = 0xC0 + Anzahl;        /* RLC-Kennung in 2-High-Bit=11 */
             RLC[1] = Pixel;                /* danach folgt der Grauwert    */
             write(PCX_fd, RLC, 2);
          }
          else {
             RLC[0] = Pixel;       /* einmaliges Vorkommen mit Grauwert<192 */
             write(PCX_fd, RLC, 1);
          }
      }
  }

  /*---------------------------- LUT noch rausschreiben --------------------*/
  RLC[0]=12;         /* Kennzeichen, dass Bild zuende und LUT jetzt beginnt */
  write(PCX_fd, RLC, 1);
  write(PCX_fd, LUT, 768);   /* Farbtabelle (LUT; Graukeil) herausschreiben */
  close(PCX_fd);

  return(TRUE);

}
