/*****************************************************************************
*                                                                            *
* void      Scan_Line_Thin(int BildNr, int Richtung)                         *
*                                                                            *
******************************************************************************
*                                                                            *
* Einfache Skelettierung durch Berechnung der Mitte einer Struktur in        *
* horizontaler oder vertikaler Richtung.                                     *
*                                                                            *
******************************************************************************
*                                                                            *
* Erstellungsdatum: 22.04.1993                                               *
* Letzte Aenderung: 22.04.1993                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Input: BildNr  = Nummer des Ausgangsbildes (1<=BildNr1<=MAX_PICTURE).      *
*        Richtung= 1: horizontal                                             *
*                  2: vertikal                                               *
*                                                                            *
******************************************************************************
*                                                                            *
* Aufruf von: ClearTextWindow(), WriteText(), Geduld(), Read_Int(),          *
*             Make_Icon(), Show_Pic()                                        *
*                                                                            *
*****************************************************************************/

#include "const.h"

#include "prototyp.h"


void Scan_Line_Thin(int BildNr, int Richtung)
{
  long z, s;              /* Zeilen- und Spaltenzaehler                     */
  long Index, Index1;    /* Anfangsadresse von Bild 1 und Bild 2           */
  long Zeilenzahl, Spaltenzahl;
  long Anfang, Ende;

  /* Titel nur anzeigen, falls keine bzw. falsche Parameter angegeben       */
  if (BildNr==FALSE) {
     ClearTextWindow(35,18,80,25);
     WriteText(35,18,"* Scan-Line-Skelettierung *");
  }

  if (BildNr == FALSE) {
     gotoxy(35,20);
     BildNr=Read_Int("Bild Nr. ", 1, ':');
  }
  /* Gueltige Bild-Nummer eingegeben? Ist dieses Bild ueberhaupt benutzt?   */
  if ((BildNr<=0) || (BildNr>MAX_PICTURE)) return;
  BildNr--;                           /* die Zaehlung faengt immer bei 0 an */
  if (!Bild[BildNr].Used) return;

  while ((Richtung!=1) && (Richtung!=2)) {
     gotoxy(35,22);
     Richtung=Read_Int("Scan-Richtung (1=horiz. 2=vert.) ", 1, ':');
  }

  Geduld();

  Index1 = (long) (BildNr) * PIC_SIZE;   /* Anfangsadresse des Bildes       */

  Spaltenzahl = Bild[BildNr].Spalten;
  Zeilenzahl  = Bild[BildNr].Zeilen;

  /* zuerst den Bildrand auf Schwarz setzen.                                */
  for (s=0; s<Spaltenzahl; s++) {
      Pictures[Index1+s]=SCHWARZ;
      Pictures[Index1+Spaltenzahl*(Zeilenzahl-1)+s]=SCHWARZ;
  }
  for (z=0; z<Zeilenzahl; z++) {
      Pictures[Index1+z*Spaltenzahl]=SCHWARZ;
      Pictures[Index1+z*Spaltenzahl+(Spaltenzahl-1)]=SCHWARZ;
  }

  if (Richtung==1) {                                         /* zeilenweise */
     for (z=0; z<Zeilenzahl; z++) {
         Anfang=0;
         Ende  =0;
         Index=Index1 + z*Spaltenzahl;
         for (s=0; s<(Spaltenzahl-1); s++) {
             if (Anfang==0) {     /* erster Schwarz-Weiss-Wechsel auffinden */
                if ((Pictures[Index+s]==SCHWARZ) && (Pictures[Index+s+1]==WEISS))
                   Anfang=s+1;
             }
             else {               /* nachfolgender Weiss-Schwarz-Wechsel    */
                if ((Pictures[Index+s]==WEISS) && (Pictures[Index+s+1]==SCHWARZ))
                   Ende=s+1;
                Pictures[Index+s]=SCHWARZ;
                if (Anfang && Ende) {
                   Pictures[Index+(Anfang+Ende)/2]=WEISS;
                   Anfang=0;
                   Ende  =0;
                }
             }
         }
     }
  }
  else {                                                    /* spaltenweise */
     for (s=0; s<Spaltenzahl; s++) {
         Anfang=0;
         Ende  =0;
         for (z=0; z<(Zeilenzahl-1); z++) {
             Index=Index1 + z*Spaltenzahl + s;
             if (Anfang==0) {     /* erster Schwarz-Weiss-Wechsel auffinden */
                if ((Pictures[Index]==SCHWARZ) && (Pictures[Index+Spaltenzahl]==WEISS))
                   Anfang=z+1;
             }
             else {               /* nachfolgender Weiss-Schwarz-Wechsel    */
                if ((Pictures[Index]==WEISS) && (Pictures[Index+Spaltenzahl]==SCHWARZ))
                   Ende=z+1;
                Pictures[Index]=SCHWARZ;
                if (Anfang && Ende) {
                   Pictures[Index1 + Spaltenzahl*(Anfang+Ende)/2 + s]=WEISS;
                   Anfang=0;
                   Ende  =0;
                }
             }
         }
     }
  }

  Make_Icon(BildNr);
  Show_Pic(BildNr, BildNr+1);

}
